/*  umidi, MIDI to URBI bridge
 *   Copyright (C) 2009  Matthieu Nottale
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 3 as
 *   published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef FACTORY_HH
#define FACTORY_HH

// Generic factory system.
#include <list>
#include <string>

template<class C> class Factory;


template<class C> inline   bool operator ==(const Factory<C>* id,
  const std::string &s) {
  return id->name==s;

}


template<class C> class Factory{
  public:
    Factory(const char * name, C*  (*factory)())
      :name(name), factory(factory),instance(0) {
        if (!itemList)
          itemList = new std::list<Factory*>;
        itemList->push_back(this);
      }

    /// Create a new instance each call
    static C * create(std::string name) {
      if (!itemList)
        return 0;
      typename std::list<Factory<C>*>::iterator it;
      it= find(itemList->begin(), itemList->end(),name);
      if (it == itemList->end())
        return 0;
      return (*(*it)->factory)();
    }

    /// Create or return a singleton
    static C * get(std::string name) {
      if (!itemList)
        return 0;
      typename std::list<Factory<C>*>::iterator it =
        std::find(itemList->begin(), itemList->end(),name);
      if (it == itemList->end())
        return 0;
      if (! (*it)->instance)
        (*it)->instance =  (*(*it)->factory)();

      return ((*it)->instance);
    }


    bool operator ==(const std::string &b) const  {return name==b;}
  private:
  std::string name;
    C * (*factory)();
    C * instance;

    static std::list<Factory<C>*> * itemList;
    template <class T> friend bool  operator ==(const Factory<T>* id,
      const std::string &s);
};



#define INSTANCIATE_FACTORY(t) template<> std::list<Factory<t>*> * Factory<t>::itemList=0

#endif
