/*  umidi, MIDI to URBI bridge
 *   Copyright (C) 2009  Matthieu Nottale
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 3 as
 *   published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MIDIIN_HH
# define MIDIIN_HH
#include <vector>
#include <iostream>
#ifdef WIN32
#include "windows.h"
#else
#include <alsa/asoundlib.h>
typedef snd_seq_t* HMIDIIN;
#endif
class MidiIn
{
public:
  enum Message {
    NOTE_OFF = 0x8,
    NOTE_ON = 0x9, // p1 pitch p2 velocity
    KEY_PRESSURE = 0xa,
    PARAMETER = 0xb, // chan is which, p1 is value
    PROGRAM = 0xc,
    CHAN_PRESSURE = 0xd,
    PITCH_WHEEL = 0xe,
    REALTIME // no parameters
  };
  // value of chan for realtime messages:
  enum Realtime {
    TICK = 0x8,  // 24 ticks per measure. click sent at start
    START = 0xa,
    CONTINUE = 0xb,
    STOP = 0xc,
    SENSE = 0xe,
    RESET = 0xf
  };
  std::vector<std::string> listDevices();
  bool open(int dev);
  void close() {}
  #ifdef WIN32
  void rawData(UINT msg, DWORD p1, DWORD p2);
  #else
  void operator()();
  #endif
  virtual void midiData(unsigned char msg, unsigned char channel,
    unsigned char v1, unsigned char v2)
  {
    dump(std::cerr, msg, channel, v1, v2);
  }
  void dump(std::ostream& o, unsigned char msg, unsigned char channel,
    unsigned char v1, unsigned char v2);
  bool debug;
  private:
    HMIDIIN handle;
};
#endif

