/*  umidi, MIDI to URBI bridge
 *   Copyright (C) 2009  Matthieu Nottale
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 3 as
 *   published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <fstream>
#include <libport/foreach.hh>
#include <libport/unistd.h>
#include <libport/assert.hh>
#include <libport/cstdio>

#include "umidi.hh"

Midi midi;

int main(int argc, const char* argv[])
{
  // FIXME: normalize, usage.
  // The midi stuff is very different between windows and linux.
  int argp = 1;
  int dev = 0;
#ifdef WIN32
  std::vector<std::string> devs = midi.listDevices();
  std::cerr <<"Midi device list:" << std::endl;
  for (int i=0; i<devs.size(); i++)
    std::cerr << i << ": " << devs[i] << std::endl;
  if (argc == 1)
    std::cerr <<"Warning, opening first available MIDI device" << std::endl;
  else
    dev = strtol(argv[argp++], 0, 0);
#endif
  if (!midi.open(dev))
  {
    libport::perror("midi open");
    return 1;
  }
  std::string confFile = "umidi.conf";
  if (argc>argp)
    confFile = argv[argp++];
  std::cerr << "parsing " << confFile << std::endl;
  std::ifstream is(confFile.c_str());
  if (!midi.parseConf(is))
  {
    std::cerr <<" parsing " << confFile << " failed" << std::endl;
    std::cerr <<"Usage: " << argv[0]
    #ifdef WIN32
    << " [MidiDeviceNumber]"
    #endif
    << " [conffile] [debuglevel]\n\tParameters defaults to 1, umidi.conf and 0"
      " respectively" << std::endl;
    return 2;
  }
  if (argc>argp)
    midi.debug = strtol(argv[argp++],0,0);
  while (true)
    usleep(900000);
}
